@@ -23,10 +23,10 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
+#include <stdlib.h>
 #include <signal.h>
 #include <time.h>
 #include <syslog.h>
@@ -39,11 +39,21 @@ static gboolean do_verbose = FALSE;	/* if we should print out debugging */
 static GSList *list = NULL;
 static gchar va_args_buffer [1025];
 
+
 /**
- * gpm_add_debug_option:
+ * gpm_debug_is_verbose:
+ **/
+gboolean
+gpm_debug_is_verbose (void)
+{
+	return do_verbose;
+}
+
+/**
+ * gpm_debug_add_option:
  **/
 void
-gpm_add_debug_option (const gchar *option)
+gpm_debug_add_option (const gchar *option)
 {
 	/* adding debug option to list */
 	list = g_slist_prepend (list, (gpointer) option);
@@ -156,17 +166,28 @@ gpm_warning_real (const gchar *func,
 }
 
 /**
- * gpm_bugzilla:
- *
- * Asks the user to consult and add to bugzilla.
+ * gpm_error_real:
  **/
 void
-gpm_bugzilla (void)
+gpm_error_real (const gchar *func,
+		const gchar *file,
+		const int    line,
+		const gchar *format, ...)
 {
-	fprintf (stderr, "%s has encountered a non-critical warning.\n"
-		 "Consult %s for any known issues or a possible fix.\n"
-		 "Please file a bug with this complete message if not present\n",
-		 "GNOME Power Manager", GPM_BUGZILLA_URL);
+	va_list args;
+
+	if (do_verbose == FALSE) {
+		return;
+	}
+
+	va_start (args, format);
+	g_vsnprintf (va_args_buffer, 1024, format, args);
+	va_end (args);
+
+	/* do extra stuff for a warning */
+	fprintf (stderr, "*** ERROR ***\n");
+	gpm_print_line (func, file, line, va_args_buffer);
+	exit (0);
 }
 
 /**
@@ -220,33 +241,13 @@ gpm_debug_init (gboolean debug)
 	openlog ("gnome-power-manager", LOG_NDELAY, LOG_USER);
 }
 
-/**
- * gpm_critical_error:
- * @content: The content to show, e.g. "No icons detected"
- *
- * Shows a gtk critical error and logs to syslog.
- * NOTE: we will lose memory, but since this program is a critical error
- * that is the least of our problems...
- **/
-void
-gpm_critical_error (const gchar *format, ...)
-{
-	va_list args;
-
-	va_start (args, format);
-	g_vsnprintf (va_args_buffer, 1024, format, args);
-	va_end (args);
-
-	g_warning (va_args_buffer);
-}
-
 /**
  * gpm_debug_shutdown:
  **/
 void
 gpm_debug_shutdown (void)
 {
-	if (! is_init) {
+	if (is_init == FALSE) {
 		return;
 	}
 
