@@ -26,9 +26,10 @@
 #include <glib/gprintf.h>
 
 #include <string.h>
-#include <time.h>
 #include <dbus/dbus-gtype-specialized.h>
 
+#include <libhal-gmanager.h>
+
 #include "gpm-ac-adapter.h"
 #include "gpm-button.h"
 #include "gpm-conf.h"
@@ -36,12 +37,13 @@
 #include "gpm-common.h"
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
-#include "gpm-hal.h"
 #include "gpm-info.h"
-#include "gpm-info-data.h"
-#include "gpm-power.h"
+#include "gpm-profile.h"
+#include "gpm-array.h"
+#include "gpm-engine.h"
 #include "gpm-stock-icons.h"
 #include "gpm-idle.h"
+#include "gpm-graph-widget.h"
 
 static void     gpm_info_class_init (GpmInfoClass *klass);
 static void     gpm_info_init       (GpmInfo      *info);
@@ -55,6 +57,10 @@ static void     gpm_info_finalize   (GObject      *object);
 	G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID))
 #define GPM_DBUS_STRUCT_INT_INT_INT (dbus_g_type_get_struct ("GValueArray", \
 	G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID))
+#define GPM_DBUS_STRUCT_INT_STRING (dbus_g_type_get_struct ("GValueArray", \
+	G_TYPE_INT, G_TYPE_STRING, G_TYPE_INVALID))
+#define GPM_DBUS_STRUCT_INT_INT_INT_STRING (dbus_g_type_get_struct ("GValueArray", \
+	G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INVALID))
 
 struct GpmInfoPrivate
 {
@@ -62,17 +68,17 @@ struct GpmInfoPrivate
 	GpmButton		*button;
 	GpmControl		*control;
 	GpmDpms			*dpms;
-	GpmHal			*hal;
 	GpmIdle			*idle;
-	GpmPower		*power;
+	GpmProfile		*profile;
+	GpmEngineCollection	*collection;
 
-	GpmInfoData		*events;
-	GpmInfoData		*rate_data;
-	GpmInfoData		*time_data;
-	GpmInfoData		*percentage_data;
-	GpmInfoData		*voltage_data;
+	GpmArray		*events;
+	GpmArray		*rate_data;
+	GpmArray		*time_data;
+	GpmArray		*percentage_data;
+	GpmArray		*voltage_data;
 
-	time_t			 start_time;
+	GTimer			*timer;
 	gboolean		 is_laptop;
 };
 
@@ -139,6 +145,250 @@ device_list_to_strv (GList *list)
 	return value;
 }
 
+/**
+ * gpm_statistics_add_data_type:
+ **/
+static void
+gpm_statistics_add_data_type (GPtrArray *array,
+			      guint colour,
+			      const gchar *description)
+{
+	GValue *value;
+
+	value = g_new0 (GValue, 1);
+	g_value_init (value, GPM_DBUS_STRUCT_INT_STRING);
+	g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_STRING));
+	dbus_g_type_struct_set (value, 0, colour, 1, description, -1);
+	g_ptr_array_add (array, g_value_get_boxed (value));
+	g_free (value);
+}
+
+/**
+ * gpm_statistics_add_event_type:
+ **/
+static void
+gpm_statistics_add_event_type (GPtrArray *array,
+			       guint id,
+			       guint colour,
+			       guint shape,
+			       const gchar *description)
+{
+	GValue *value;
+
+	value = g_new0 (GValue, 1);
+	g_value_init (value, GPM_DBUS_STRUCT_INT_INT_INT_STRING);
+	g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_INT_INT_STRING));
+	dbus_g_type_struct_set (value, 0, id, 1, colour, 2, shape, 3, description, -1);
+	g_ptr_array_add (array, g_value_get_boxed (value));
+	g_free (value);
+}
+
+/**
+ * gpm_statistics_add_event_type:
+ **/
+static void
+gpm_statistics_add_events_typical (GPtrArray *array)
+{
+	/* add the general key items, TODO specify which ones make sense */
+	gpm_statistics_add_event_type (array, GPM_EVENT_ON_AC,
+				       GPM_COLOUR_BLUE,
+				       GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
+				       _("On AC"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_ON_BATTERY,
+				       GPM_COLOUR_DARK_BLUE,
+				       GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
+				       _("On battery"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_SESSION_IDLE,
+				       GPM_COLOUR_YELLOW,
+				       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				       _("Session idle"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_SESSION_ACTIVE,
+				       GPM_COLOUR_DARK_YELLOW,
+				       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				       _("Session active"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_SUSPEND,
+				       GPM_COLOUR_RED,
+				       GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+				       _("Suspend"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_RESUME,
+				       GPM_COLOUR_DARK_RED,
+				       GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+				       _("Resume"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_HIBERNATE,
+				       GPM_COLOUR_MAGENTA,
+				       GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+				       _("Hibernate"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_LID_CLOSED,
+				       GPM_COLOUR_GREEN,
+				       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				       _("Lid closed"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_LID_OPENED,
+				       GPM_COLOUR_DARK_GREEN,
+				       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				       _("Lid opened"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_NOTIFICATION,
+				       GPM_COLOUR_GREY,
+				       GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
+				       _("Notification"));
+#ifdef HAVE_DPMS_EXTENSION
+	gpm_statistics_add_event_type (array, GPM_EVENT_DPMS_ON,
+				       GPM_COLOUR_CYAN,
+				       GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
+				       _("DPMS On"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_DPMS_STANDBY,
+				       GPM_COLOUR_CYAN,
+				       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				       _("DPMS Standby"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_DPMS_SUSPEND,
+				       GPM_COLOUR_CYAN,
+				       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				       _("DPMS Suspend"));
+	gpm_statistics_add_event_type (array, GPM_EVENT_DPMS_OFF,
+				       GPM_COLOUR_CYAN,
+				       GPM_GRAPH_WIDGET_SHAPE_DIAMOND,
+				       _("DPMS Off"));
+#endif
+}
+
+/**
+ * gpm_statistics_get_parameters:
+ **/
+gboolean
+gpm_statistics_get_parameters (GpmInfo   *info,
+			       gchar	 *type,
+			       gchar	 **axis_type_x,
+			       gchar	 **axis_type_y,
+			       gchar	 **axis_desc_x,
+			       gchar	 **axis_desc_y,
+			       GPtrArray **key_data,
+			       GPtrArray **key_event,
+			       GError	 **error)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
+	g_return_val_if_fail (type != NULL, FALSE);
+	g_return_val_if_fail (axis_type_x != NULL, FALSE);
+	g_return_val_if_fail (axis_type_y != NULL, FALSE);
+	g_return_val_if_fail (axis_desc_x != NULL, FALSE);
+	g_return_val_if_fail (axis_desc_y != NULL, FALSE);
+	g_return_val_if_fail (key_data != NULL, FALSE);
+	g_return_val_if_fail (key_event != NULL, FALSE);
+
+	*key_data = g_ptr_array_new ();
+	*key_event = g_ptr_array_new ();
+
+	if (strcmp (type, "power") == 0) {
+		*axis_type_x = g_strdup ("time");
+		*axis_type_y = g_strdup ("power");
+		*axis_desc_x = g_strdup (_("Time since startup"));
+		*axis_desc_y = g_strdup (_("Power (mWh)"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_RED, _("Power"));
+		gpm_statistics_add_events_typical (*key_event);
+		return TRUE;
+	}
+	if (strcmp (type, "time") == 0) {
+		*axis_type_x = g_strdup ("time");
+		*axis_type_y = g_strdup ("time");
+		*axis_desc_x = g_strdup (_("Time since startup"));
+		*axis_desc_y = g_strdup (_("Estimated time"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_RED, _("Time"));
+		gpm_statistics_add_events_typical (*key_event);
+		return TRUE;
+	}
+	if (strcmp (type, "charge") == 0) {
+		*axis_type_x = g_strdup ("time");
+		*axis_type_y = g_strdup ("percentage");
+		*axis_desc_x = g_strdup (_("Time since startup"));
+		*axis_desc_y = g_strdup (_("Battery percentage"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_RED, _("Percentage"));
+		gpm_statistics_add_events_typical (*key_event);
+		return TRUE;
+	}
+	if (strcmp (type, "voltage") == 0) {
+		*axis_type_x = g_strdup ("time");
+		*axis_type_y = g_strdup ("voltage");
+		*axis_desc_x = g_strdup (_("Time since startup"));
+		*axis_desc_y = g_strdup (_("Battery Voltage"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_GREEN, _("Voltage"));
+		gpm_statistics_add_events_typical (*key_event);
+		return TRUE;
+	}
+	if (strcmp (type, "profile-charge-accuracy") == 0) {
+		*axis_type_x = g_strdup ("percentage");
+		*axis_type_y = g_strdup ("percentage");
+		*axis_desc_x = g_strdup (_("Battery percentage"));
+		*axis_desc_y = g_strdup (_("Accuracy of reading"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_BLUE, _("Valid data"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_DARK_BLUE, _("No data"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_HIBERNATE,
+					       GPM_COLOUR_CYAN,
+					       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				 	      _("Charge Void"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_SUSPEND,
+					       GPM_COLOUR_YELLOW,
+					       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				 	      _("Discharge Void"));
+		return TRUE;
+	}
+	if (strcmp (type, "profile-charge-time") == 0) {
+		*axis_type_x = g_strdup ("percentage");
+		*axis_type_y = g_strdup ("time");
+		*axis_desc_x = g_strdup (_("Battery percentage"));
+		*axis_desc_y = g_strdup (_("Average time elapsed"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_BLUE, _("Valid data"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_DARK_BLUE, _("No data"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_HIBERNATE,
+					       GPM_COLOUR_CYAN,
+					       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				 	      _("Charge Void"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_SUSPEND,
+					       GPM_COLOUR_YELLOW,
+					       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				 	      _("Discharge Void"));
+		return TRUE;
+	}
+	if (strcmp (type, "profile-discharge-accuracy") == 0) {
+		*axis_type_x = g_strdup ("percentage");
+		*axis_type_y = g_strdup ("percentage");
+		*axis_desc_x = g_strdup (_("Battery percentage"));
+		*axis_desc_y = g_strdup (_("Accuracy of reading"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_RED, _("Valid data"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_DARK_RED, _("No data"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_HIBERNATE,
+					       GPM_COLOUR_CYAN,
+					       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				 	      _("Charge Void"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_SUSPEND,
+					       GPM_COLOUR_YELLOW,
+					       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				 	      _("Discharge Void"));
+		return TRUE;
+	}
+	if (strcmp (type, "profile-discharge-time") == 0) {
+		*axis_type_x = g_strdup ("percentage");
+		*axis_type_y = g_strdup ("time");
+		*axis_desc_x = g_strdup (_("Battery percentage"));
+		*axis_desc_y = g_strdup (_("Average time elapsed"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_RED, _("Valid data"));
+		gpm_statistics_add_data_type (*key_data, GPM_COLOUR_DARK_RED, _("No data"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_HIBERNATE,
+					       GPM_COLOUR_CYAN,
+					       GPM_GRAPH_WIDGET_SHAPE_SQUARE,
+				 	      _("Charge Void"));
+		gpm_statistics_add_event_type (*key_event, GPM_EVENT_SUSPEND,
+					       GPM_COLOUR_YELLOW,
+					       GPM_GRAPH_WIDGET_SHAPE_TRIANGLE,
+				 	      _("Discharge Void"));
+		return TRUE;
+	}
+
+	/* not recognised... */
+	*error = g_error_new (gpm_info_error_quark (),
+			      GPM_INFO_ERROR_INVALID_TYPE,
+			      "Invalid type %s", type);
+	return FALSE;
+}
+
 /**
  * gpm_statistics_get_types:
  * @info: This class instance
@@ -152,29 +402,45 @@ gpm_statistics_get_data_types (GpmInfo  *info,
 			       GError  **error)
 {
 	GList *list = NULL;
-	GList *data;
+	GpmArray *array;
 
 	g_return_val_if_fail (info != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
 	g_return_val_if_fail (types != NULL, FALSE);
 
 	/* only return the type if we have sufficient data */
-	data = gpm_info_data_get_list (info->priv->rate_data);
-	if (g_list_length (data) > 2) {
+	array = info->priv->rate_data;
+	if (array != NULL && gpm_array_get_size (array) > 2) {
 		list = g_list_append (list, "power");
 	}
-	data = gpm_info_data_get_list (info->priv->time_data);
-	if (g_list_length (data) > 2) {
+	array = info->priv->time_data;
+	if (array != NULL && gpm_array_get_size (array) > 2) {
 		list = g_list_append (list, "time");
 	}
-	data = gpm_info_data_get_list (info->priv->percentage_data);
-	if (g_list_length (data) > 2) {
+	array = info->priv->percentage_data;
+	if (array != NULL && gpm_array_get_size (array) > 2) {
 		list = g_list_append (list, "charge");
 	}
-	data = gpm_info_data_get_list (info->priv->voltage_data);
-	if (g_list_length (data) > 2) {
+	array = info->priv->voltage_data;
+	if (array != NULL && gpm_array_get_size (array) > 2) {
 		list = g_list_append (list, "voltage");
 	}
+	array = gpm_profile_get_data_accuracy_percent (info->priv->profile, FALSE);
+	if (array != NULL && gpm_array_get_size (array) > 2) {
+		list = g_list_append (list, "profile-charge-accuracy");
+	}
+	array = gpm_profile_get_data_time_percent (info->priv->profile, FALSE);
+	if (array != NULL && gpm_array_get_size (array) > 2) {
+		list = g_list_append (list, "profile-charge-time");
+	}
+	array = gpm_profile_get_data_accuracy_percent (info->priv->profile, TRUE);
+	if (array != NULL && gpm_array_get_size (array) > 2) {
+		list = g_list_append (list, "profile-discharge-accuracy");
+	}
+	array = gpm_profile_get_data_time_percent (info->priv->profile, TRUE);
+	if (array != NULL && gpm_array_get_size (array) > 2) {
+		list = g_list_append (list, "profile-discharge-time");
+	}
 
 	*types = device_list_to_strv (list);
 
@@ -182,90 +448,38 @@ gpm_statistics_get_data_types (GpmInfo  *info,
 	return TRUE;
 }
 
-/**
- * gpm_statistics_get_axis_types:
- * @info: This class instance
- * @type: The graph type, e.g. "charge", "power", "time", etc.
- * @axis_type_x: The axis type, only "percentage", "power" or "time"
- * @axis_type_y: The axis type, only "percentage", "power" or "time"
- *
- * Return value: TRUE for success, if FALSE then error set
- **/
-gboolean
-gpm_statistics_get_axis_types (GpmInfo *info,
-			       gchar   *type,
-			       gchar  **axis_type_x,
-			       gchar  **axis_type_y,
-			       GError **error)
-{
-	g_return_val_if_fail (info != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
-	g_return_val_if_fail (type != NULL, FALSE);
-	g_return_val_if_fail (axis_type_x != NULL, FALSE);
-	g_return_val_if_fail (axis_type_y != NULL, FALSE);
-
-	if (strcmp (type, "power") == 0) {
-		*axis_type_x = g_strdup ("time");
-		*axis_type_y = g_strdup ("power");
-		return TRUE;
-	}
-	if (strcmp (type, "time") == 0) {
-		*axis_type_x = g_strdup ("time");
-		*axis_type_y = g_strdup ("time");
-		return TRUE;
-	}
-	if (strcmp (type, "charge") == 0) {
-		*axis_type_x = g_strdup ("time");
-		*axis_type_y = g_strdup ("percentage");
-		return TRUE;
-	}
-	if (strcmp (type, "voltage") == 0) {
-		*axis_type_x = g_strdup ("time");
-		*axis_type_y = g_strdup ("voltage");
-		return TRUE;
-	}
-	
-	/* not recognised... */
-	*error = g_error_new (gpm_info_error_quark (),
-			      GPM_INFO_ERROR_INVALID_TYPE,
-			      "Invalid type %s", type);
-	return FALSE;
-}
-
 /**
  * gpm_statistics_get_event_log:
  * @info: This class instance
- * @seconds: The amount of data to get, currently unused.
  *
  * Return value: TRUE for success
  **/
 gboolean
 gpm_statistics_get_event_log (GpmInfo    *info,
-			      gint 	  seconds,
 			      GPtrArray **array,
 			      GError    **error)
 {
-	GList *events, *l;
-	GpmInfoDataPoint *new;
+	GpmArrayPoint *point;
+	GpmArray *events;
+	gint i;
 	GValue *value;
+	guint length;
 
 	g_return_val_if_fail (info != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
-	g_return_val_if_fail (array != NULL, FALSE);
 
-	events = gpm_info_data_get_list (info->priv->events);
-	*array = g_ptr_array_sized_new (g_list_length (events));
-
-	for (l=events; l != NULL; l=l->next) {
-		new = (GpmInfoDataPoint *) l->data;
+	events = info->priv->events;
+	length = gpm_array_get_size (events);
+	*array = g_ptr_array_sized_new (length);
+	for (i=0; i < length; i++) {
+		point = gpm_array_get (events, i);
 		value = g_new0 (GValue, 1);
 		g_value_init (value, GPM_DBUS_STRUCT_INT_INT);
 		g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_INT));
-		dbus_g_type_struct_set (value, 0, new->time, 1, new->value, -1);
+		dbus_g_type_struct_set (value, 0, point->x, 1, point->y, -1);
 		g_ptr_array_add (*array, g_value_get_boxed (value));
 		g_free (value);
 	}
-	g_list_free (events);
 
 	return TRUE;
 }
@@ -273,26 +487,25 @@ gpm_statistics_get_event_log (GpmInfo    *info,
 /**
  * gpm_statistics_get_data:
  * @info: This class instance
- * @seconds: The amount of data to get, currently unused.
  * @type: The graph type, e.g. "charge", "power", "time", etc.
  *
  * Return value: TRUE for success
  **/
 gboolean
 gpm_statistics_get_data (GpmInfo     *info,
-			 gint 	      seconds,
 			 const gchar *type,
 			 GPtrArray  **array,
 			 GError	    **error)
 {
-	GList *events, *l;
-	GpmInfoDataPoint *new;
 	GValue *value;
+	GpmArray *events;
+	GpmArrayPoint *point;
+	gint i;
+	guint length;
 
 	g_return_val_if_fail (info != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
 	g_return_val_if_fail (type != NULL, FALSE);
-	g_return_val_if_fail (array != NULL, FALSE);
 
 	if (info->priv->is_laptop == FALSE) {
 		gpm_warning ("Data not available as not a laptop");
@@ -303,13 +516,21 @@ gpm_statistics_get_data (GpmInfo     *info,
 	}
 
 	if (strcmp (type, "power") == 0) {
-		events = gpm_info_data_get_list (info->priv->rate_data);
+		events = info->priv->rate_data;
 	} else if (strcmp (type, "time") == 0) {
-		events = gpm_info_data_get_list (info->priv->time_data);
+		events = info->priv->time_data;
 	} else if (strcmp (type, "charge") == 0) {
-		events = gpm_info_data_get_list (info->priv->percentage_data);
+		events = info->priv->percentage_data;
 	} else if (strcmp (type, "voltage") == 0) {
-		events = gpm_info_data_get_list (info->priv->voltage_data);
+		events = info->priv->voltage_data;
+	} else if (strcmp (type, "profile-charge-accuracy") == 0) {
+		events = gpm_profile_get_data_accuracy_percent (info->priv->profile, FALSE);
+	} else if (strcmp (type, "profile-charge-time") == 0) {
+		events = gpm_profile_get_data_time_percent (info->priv->profile, FALSE);
+	} else if (strcmp (type, "profile-discharge-accuracy") == 0) {
+		events = gpm_profile_get_data_accuracy_percent (info->priv->profile, TRUE);
+	} else if (strcmp (type, "profile-discharge-time") == 0) {
+		events = gpm_profile_get_data_time_percent (info->priv->profile, TRUE);
 	} else {
 		gpm_warning ("Data type %s not known!", type);
 		*error = g_error_new (gpm_info_error_quark (),
@@ -324,22 +545,19 @@ gpm_statistics_get_data (GpmInfo     *info,
 				      GPM_INFO_ERROR_DATA_NOT_AVAILABLE,
 				      "Data not available");
 		return FALSE;
-	}		
+	}
 
-	/* TODO: process seconds */
-	*array = g_ptr_array_sized_new (g_list_length (events));
-
-	for (l=events; l != NULL; l=l->next) {
-		new = (GpmInfoDataPoint *) l->data;
+	length = gpm_array_get_size (events);
+	*array = g_ptr_array_sized_new (length);
+	for (i=0; i < length; i++) {
+		point = gpm_array_get (events, i);
 		value = g_new0 (GValue, 1);
 		g_value_init (value, GPM_DBUS_STRUCT_INT_INT_INT);
 		g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_INT_INT));
-		dbus_g_type_struct_set (value, 0, new->time, 1, new->value, 2, new->colour, -1);
+		dbus_g_type_struct_set (value, 0, point->x, 1, point->y, 2, point->data, -1);
 		g_ptr_array_add (*array, g_value_get_boxed (value));
 		g_free (value);
 	}
-
-	g_list_free (events);
 	return TRUE;
 }
 
@@ -360,11 +578,25 @@ gpm_info_event_log (GpmInfo	       *info,
 	g_return_if_fail (GPM_IS_INFO (info));
 	gpm_debug ("Adding %i to the event log", event);
 
-	gpm_info_data_add_always (info->priv->events,
-				  time (NULL) - info->priv->start_time,
-				  event, 0, desc);
+	gpm_array_append (info->priv->events,
+			  g_timer_elapsed (info->priv->timer, NULL), event, 0);
 }
 
+/**
+ * gpm_info_set_collection_data:
+ **/
+gboolean
+gpm_info_set_collection_data (GpmInfo             *info,
+			      GpmEngineCollection *collection)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_INFO (info), FALSE);
+
+	info->priv->collection = collection;
+	return TRUE;
+}
+
+
 /**
  * gpm_info_log_do_poll:
  * @data: gpointer to this info class instance
@@ -376,66 +608,59 @@ static gboolean
 gpm_info_log_do_poll (gpointer data)
 {
 	GpmInfo *info = (GpmInfo*) data;
+	GpmCellArray *array;
+	GpmCellUnit *unit;
 
 	int value_x;
 	int colour;
 
-	GpmPowerStatus battery_status;
-	gpm_power_get_battery_status (info->priv->power,
-				      GPM_POWER_KIND_PRIMARY,
-				      &battery_status);
+	array = info->priv->collection->primary;
+	unit = gpm_cell_array_get_unit (array);
 
 	if (info->priv->is_laptop == TRUE) {
 		/* work out seconds elapsed */
-		value_x = time (NULL) - (info->priv->start_time + GPM_INFO_DATA_POLL);
+		value_x = g_timer_elapsed (info->priv->timer, NULL) - GPM_INFO_DATA_POLL;
 
 		/* set the correct colours */
-		if (battery_status.is_discharging) {
-			colour = GPM_GRAPH_WIDGET_COLOUR_DISCHARGING;
-		} else if (battery_status.is_charging) {
-			colour = GPM_GRAPH_WIDGET_COLOUR_CHARGING;
+		if (unit->is_discharging) {
+			colour = GPM_COLOUR_DISCHARGING;
+		} else if (unit->is_charging) {
+			colour = GPM_COLOUR_CHARGING;
 		} else {
-			colour = GPM_GRAPH_WIDGET_COLOUR_CHARGED;
+			colour = GPM_COLOUR_CHARGED;
 		}
 
-		gpm_info_data_add (info->priv->percentage_data,
-				   value_x,
-				   battery_status.percentage_charge, colour);
+		gpm_array_add (info->priv->percentage_data, value_x,
+			       unit->percentage, colour);
 
 		/* sanity check to less than 100W */
-		if (battery_status.charge_rate_raw < 100000) {
-			gpm_info_data_add (info->priv->rate_data,
-					   value_x,
-					   battery_status.charge_rate_raw, colour);
+		if (unit->rate < 100000) {
+			gpm_array_add (info->priv->rate_data, value_x,
+				       unit->rate, colour);
 		}
 
 		/* sanity check to less than 10 hours */
-		if (battery_status.remaining_time < 10*60*60) {
-			gpm_info_data_add (info->priv->time_data,
-					   value_x,
-					   battery_status.remaining_time, colour);
+		if (unit->time_discharge < 10*60*60) {
+			gpm_array_add (info->priv->time_data, value_x,
+					   unit->time_discharge, colour);
 		}
-		gpm_info_data_add (info->priv->voltage_data,
-				   value_x,
-				   battery_status.voltage, colour);
+		gpm_array_add (info->priv->voltage_data, value_x,
+			       unit->voltage, colour);
 	}
 	return TRUE;
 }
 
 /**
- * power_on_ac_changed_cb:
- * @power: The power class instance
- * @on_ac: if we are on AC power
- * @manager: This class instance
+ * ac_adapter_changed_cb:
  *
- * Does the actions when the ac power source is inserted/removed.
+ * Does the actions when the ac source is inserted/removed.
  **/
 static void
-ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
-		       GpmAcAdapterState state,
-		       GpmInfo          *info)
+ac_adapter_changed_cb (GpmAcAdapter *ac_adapter,
+		       gboolean      on_ac,
+		       GpmInfo      *info)
 {
-	if (state == GPM_AC_ADAPTER_PRESENT) {
+	if (on_ac == TRUE) {
 		gpm_info_event_log (info, GPM_EVENT_ON_AC,
 				    _("AC adapter inserted"));
 	} else {
@@ -446,13 +671,12 @@ ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
 
 /**
  * button_pressed_cb:
- * @power: The power class instance
  * @type: The button type, e.g. "power"
  * @state: The state, where TRUE is depressed or closed
  * @brightness: This class instance
  **/
 static void
-button_pressed_cb (GpmButton   *power,
+button_pressed_cb (GpmButton   *button,
 		   const gchar *type,
 		   GpmInfo     *info)
 {
@@ -575,12 +799,11 @@ gpm_info_class_init (GpmInfoClass *klass)
 static void
 gpm_info_init (GpmInfo *info)
 {
+	HalGManager *hal_manager;
 	info->priv = GPM_INFO_GET_PRIVATE (info);
 
 	/* record our start time */
-	info->priv->start_time = time (NULL);
-
-	info->priv->hal = gpm_hal_new ();
+	info->priv->timer = g_timer_new ();
 
 	info->priv->control = gpm_control_new ();
 	g_signal_connect (info->priv->control, "resume",
@@ -589,10 +812,12 @@ gpm_info_init (GpmInfo *info)
 			  G_CALLBACK (control_sleep_failure_cb), info);
 
 	/* find out if we should log and display the extra graphs */
-	info->priv->is_laptop = gpm_hal_is_laptop (info->priv->hal);
+	hal_manager = hal_gmanager_new ();
+	info->priv->is_laptop = hal_gmanager_is_laptop (hal_manager);
+	g_object_unref (hal_manager);
 
-	/* singleton, so okay */
-	info->priv->power = gpm_power_new ();
+	/* set default, we have to set these from the manager */
+	info->priv->profile = gpm_profile_new ();
 
 	/* we use ac_adapter so we can log the event */
 	info->priv->ac_adapter = gpm_ac_adapter_new ();
@@ -615,12 +840,12 @@ gpm_info_init (GpmInfo *info)
 			  G_CALLBACK (dpms_mode_changed_cb), info);
 
 	/* set to a blank list */
-	info->priv->events = gpm_info_data_new ();
+	info->priv->events = gpm_array_new ();
 	if (info->priv->is_laptop == TRUE) {
-		info->priv->percentage_data = gpm_info_data_new ();
-		info->priv->rate_data = gpm_info_data_new ();
-		info->priv->time_data = gpm_info_data_new ();
-		info->priv->voltage_data = gpm_info_data_new ();
+		info->priv->percentage_data = gpm_array_new ();
+		info->priv->rate_data = gpm_array_new ();
+		info->priv->time_data = gpm_array_new ();
+		info->priv->voltage_data = gpm_array_new ();
 
 		/* set up the timer callback so we can log data */
 		g_timeout_add (GPM_INFO_DATA_POLL * 1000, gpm_info_log_do_poll, info);
@@ -639,11 +864,11 @@ gpm_info_init (GpmInfo *info)
 		gpm_conf_get_uint (conf, GPM_CONF_GRAPH_DATA_MAX_TIME, &max_time);
 		g_object_unref (conf);
 
-		gpm_info_data_set_max_time (info->priv->events, max_time);
-		gpm_info_data_set_max_time (info->priv->percentage_data, max_time);
-		gpm_info_data_set_max_time (info->priv->rate_data, max_time);
-		gpm_info_data_set_max_time (info->priv->time_data, max_time);
-		gpm_info_data_set_max_time (info->priv->voltage_data, max_time);
+		gpm_array_set_max_width (info->priv->events, max_time);
+		gpm_array_set_max_width (info->priv->percentage_data, max_time);
+		gpm_array_set_max_width (info->priv->rate_data, max_time);
+		gpm_array_set_max_width (info->priv->time_data, max_time);
+		gpm_array_set_max_width (info->priv->voltage_data, max_time);
 	}
 }
 
@@ -673,21 +898,18 @@ gpm_info_finalize (GObject *object)
 	if (info->priv->voltage_data) {
 		g_object_unref (info->priv->voltage_data);
 	}
-	if (info->priv->ac_adapter != NULL) {
-		g_object_unref (info->priv->ac_adapter);
-	}
-	if (info->priv->idle != NULL) {
-		g_object_unref (info->priv->idle);
-	}
 	if (info->priv->dpms != NULL) {
 		g_object_unref (info->priv->dpms);
 	}
 	if (info->priv->control != NULL) {
 		g_object_unref (info->priv->control);
 	}
+	g_object_unref (info->priv->ac_adapter);
+	g_object_unref (info->priv->idle);
 	g_object_unref (info->priv->events);
-	g_object_unref (info->priv->power);
-	g_object_unref (info->priv->hal);
+	g_object_unref (info->priv->profile);
+
+	g_timer_destroy (info->priv->timer);
 
 	G_OBJECT_CLASS (gpm_info_parent_class)->finalize (object);
 }
@@ -703,3 +925,4 @@ gpm_info_new (void)
 	info = g_object_new (GPM_TYPE_INFO, NULL);
 	return GPM_INFO (info);
 }
+
