@@ -29,7 +29,7 @@
 #include "gpm-conf.h"
 #include "gpm-screensaver.h"
 #include "gpm-debug.h"
-#include "gpm-proxy.h"
+#include <libdbus-proxy.h>
 
 static void     gpm_screensaver_class_init (GpmScreensaverClass *klass);
 static void     gpm_screensaver_init       (GpmScreensaver      *screensaver);
@@ -39,7 +39,7 @@ static void     gpm_screensaver_finalize   (GObject		*object);
 
 struct GpmScreensaverPrivate
 {
-	GpmProxy		*gproxy;
+	DbusProxy		*gproxy;
 	GpmConf			*conf;
 	guint			 idle_delay;	/* the setting in g-s-p, cached */
 };
@@ -52,7 +52,8 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint signals [LAST_SIGNAL] = { 0, };
+static gpointer gpm_screensaver_object = NULL;
 
 G_DEFINE_TYPE (GpmScreensaver, gpm_screensaver, G_TYPE_OBJECT)
 
@@ -101,11 +102,11 @@ gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	g_signal_emit (screensaver, signals [CONNECTION_CHANGED], 0, TRUE);
 
@@ -220,11 +221,11 @@ gpm_screensaver_lock (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	gpm_debug ("doing gnome-screensaver lock");
 	dbus_g_proxy_call_no_reply (proxy, "Lock", G_TYPE_INVALID);
@@ -266,11 +267,11 @@ gpm_screensaver_add_throttle (GpmScreensaver *screensaver,
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 	g_return_val_if_fail (reason != NULL, 0);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	/* shouldn't be, but make sure proxy valid */
 	if (proxy == NULL) {
@@ -308,11 +309,11 @@ gpm_screensaver_remove_throttle (GpmScreensaver *screensaver,
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	gpm_debug ("removing throttle: id %u", cookie);
 	ret = dbus_g_proxy_call (proxy, "UnThrottle", &error,
@@ -347,11 +348,11 @@ gpm_screensaver_check_running (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	ret = dbus_g_proxy_call (proxy, "GetActive", &error,
 				 G_TYPE_INVALID,
@@ -380,11 +381,11 @@ gpm_screensaver_poke (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	gpm_debug ("poke");
 	dbus_g_proxy_call_no_reply (proxy,
@@ -409,11 +410,11 @@ gpm_screensaver_get_idle (GpmScreensaver *screensaver, gint *time_secs)
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 	g_return_val_if_fail (time != NULL, FALSE);
 
-	proxy = gpm_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	ret = dbus_g_proxy_call (proxy, "GetActiveTime", &error,
 				 G_TYPE_INVALID,
@@ -515,9 +516,9 @@ gpm_screensaver_init (GpmScreensaver *screensaver)
 
 	screensaver->priv = GPM_SCREENSAVER_GET_PRIVATE (screensaver);
 
-	screensaver->priv->gproxy = gpm_proxy_new ();
-	proxy = gpm_proxy_assign (screensaver->priv->gproxy,
-				  GPM_PROXY_SESSION,
+	screensaver->priv->gproxy = dbus_proxy_new ();
+	proxy = dbus_proxy_assign (screensaver->priv->gproxy,
+				  DBUS_PROXY_SESSION,
 				  GS_LISTENER_SERVICE,
 				  GS_LISTENER_PATH,
 				  GS_LISTENER_INTERFACE);
@@ -565,13 +566,11 @@ gpm_screensaver_finalize (GObject *object)
 GpmScreensaver *
 gpm_screensaver_new (void)
 {
-	static GpmScreensaver *screensaver = NULL;
-
-	if (screensaver != NULL) {
-		g_object_ref (screensaver);
-		return screensaver;
+	if (gpm_screensaver_object != NULL) {
+		g_object_ref (gpm_screensaver_object);
+	} else {
+		gpm_screensaver_object = g_object_new (GPM_TYPE_SCREENSAVER, NULL);
+		g_object_add_weak_pointer (gpm_screensaver_object, &gpm_screensaver_object);
 	}
-
-	screensaver = g_object_new (GPM_TYPE_SCREENSAVER, NULL);
-	return screensaver;
+	return GPM_SCREENSAVER (gpm_screensaver_object);
 }
